@@ -1,10 +1,12 @@
 import abc
 from dataclasses import dataclass
-from typing import Dict, List, Union
+from typing import Dict, Union
 
 import numpy as np
 from class_registry import AutoRegister
 from numpy.typing import ArrayLike
+from IPython.display import Markdown, Math, display
+import pandas as pd
 
 from popsynth.utils.configuration import popsynth_config
 from popsynth.utils.logging import setup_logger
@@ -28,15 +30,14 @@ class Distribution(object,
 
     def __init__(self, name: str, seed: int, form: str) -> None:
         """
-        A distribution base class
-
-        :param name: the name of the distribution
-        :param seed: the random seed
-        :param form: the latex form
-        :param truth: dictionary holding true parameters
-        :returns:
-        :rtype:
+        A distribution base class.
 
+        :param name: Name of the distribution
+        :type name: str
+        :param seed: Random seed
+        :type seed: int
+        :param form: the LaTeX form
+        :type form: str
         """
 
         self._parameter_storage: Dict[str, float] = {}
@@ -47,19 +48,38 @@ class Distribution(object,
 
     @property
     def name(self) -> str:
+        """
+        The name of the distribution
+
+        :returns: 
+
+        """
         return self._name
 
     @property
     def form(self) -> str:
+        """
+        The latex form of the 
+        distribution
+
+        :returns: 
+
+        """
         return self._form
 
     @property
     def params(self) -> Dict[str, float]:
+        """
+        The parameters of the distribution
+
+        """
         return self._parameter_storage
 
     @property
     def truth(self) -> Dict[str, float]:
-
+        """
+        value of the parameters used to simulate
+        """
         out = {}
 
         for k, v in self._parameter_storage.items():
@@ -70,9 +90,41 @@ class Distribution(object,
 
         return out
 
+    def display(self):
+        """
+        use ipython pretty display to 
+        display the functions
+
+        :returns: 
+
+        """
+
+        out = {"parameter": [], "value": []}
+
+        for k, v in self.params.items():
+
+            out["parameter"].append(k)
+            out["value"].append(v)
+
+        display(Math(self._form))
+        display(pd.DataFrame(out))
+
+    def __repr__(self):
+
+        out = f"{self._name}\n"
+        out += f"{self._form}\n"
+
+        for k, v in self.params.items():
+            out += f"{k}: {v}\n"
+
+        return out
+
 
 @dataclass
 class SpatialContainer:
+    """
+    Container for 3D spatial values.
+    """
 
     distance: ArrayLike
     theta: ArrayLike
@@ -94,14 +146,15 @@ class SpatialDistribution(Distribution):
 
     def __init__(self, name: str, seed: int, form: Union[str, None] = None):
         """
-        A spatial distribution such as a redshift
-        distribution
-
-        :param name: the name of the distribution
-        :param r_max: the maximum distance to sample
-        :param seed: the random seed
-        :param form: the latex form
+        A base class for spatial distributions,
+        such as redshift distributions.
 
+        :param name: Name of the distribution
+        :type name: str
+        :param seed: Random seed
+        :type seed: int
+        :param form: the LaTeX form
+        :type form: Union[str, None]
         """
         self._theta = None
         self._phi = None
@@ -113,70 +166,128 @@ class SpatialDistribution(Distribution):
     @abc.abstractmethod
     def differential_volume(self, distance):
         """
-        the differential volume
-
-        :param distance:
-        :returns:
-        :rtype:
+        The differential volume
 
+        :param distance: Distance
         """
 
         raise RuntimeError("Must be implemented in derived class")
 
     @abc.abstractmethod
     def dNdV(self, distance):
+        """
+        The differential number of objects
+        per volume element
 
+        :param distance: 
+        :type distance: 
+        :returns: 
+
+        """
         raise RuntimeError("Must be implemented in derived class")
         pass
 
-    def time_adjustment(self, r):
+    def time_adjustment(self, distance):
         """
-        the time adjustment
-
-        :param r:
-        :returns:
-        :rtype:
+        The time adjustment
 
+        :param distance: Distance
         """
 
         return 1.0
 
     @abc.abstractmethod
     def transform(self, flux, distance):
+        """
+        The transform from luminosity to flux
+        for the 
+
+        :param flux: 
+        :type flux: 
+        :param distance: 
+        :type distance: 
+        :returns: 
+
+        """
         pass
 
     @property
     def theta(self) -> np.ndarray:
+        """
+        the polar coordinate of the objects
+
+        :returns: 
+
+        """
         return self._theta
 
     @property
     def phi(self) -> np.ndarray:
+        """
+        the longitudinal coordinate fo the objects
+
+        :returns: 
+
+        """
+
         return self._phi
 
     @property
     def dec(self) -> np.ndarray:
+        """
+        The declination of the objects
+
+        :returns: 
+
+        """
         return 90 - np.rad2deg(self._theta)
 
     @property
     def ra(self) -> np.ndarray:
+        """
+        the right acension of the objects
+
+        :returns: 
+
+        """
         return np.rad2deg(self._phi)
 
     @property
     def distances(self) -> np.ndarray:
+        """
+        the distances to the objects
+
+        :returns: 
+
+        """
         return self._distances
 
     @property
     def spatial_values(self):
+        """
+        All the spatial values of the objects
+        :returns: 
 
+        """
         return SpatialContainer(self._distances, self._theta, self._phi)
 
     def draw_sky_positions(self, size: int) -> None:
+        """
+        Draw teh sky positions of the objects
 
+        :param size: 
+        :type size: int
+        :returns: 
+
+        """
         self._theta, self._phi = sample_theta_phi(size)
 
     def draw_distance(self, size: int) -> None:
         """
-        Draw the distances from the specified dN/dr model
+        Draw the distances from the specified dN/dr model.
+
+        :param size: Number of distances to sample
+        :type size: int
         """
 
         # create a callback for the sampler
@@ -225,12 +336,14 @@ class LuminosityDistribution(Distribution):
 
     def __init__(self, name: str, seed: int, form: Union[str, None] = None):
         """
-        A luminosity distribution such as a
-        distribution
+        A base class for luminosity distributions.
 
-        :param name: the name of the distribution
-        :param seed: the random seed
-        :param form: the latex form
+        :param name: Name of the distribution
+        :type name: str
+        :param seed: Random seed
+        :type seed: int
+        :param form: the LaTeX form
+        :type form: Union[str, None]
         """
 
         super(LuminosityDistribution, self).__init__(
@@ -242,16 +355,22 @@ class LuminosityDistribution(Distribution):
     @abc.abstractmethod
     def phi(self, luminosity):
         """
-        The functional form of the distribution
-
-        :param L:
-        :returns:
-        :rtype:
-
+        The functional form of the distribution.
+        not required for sampling
+        :param luminosity: Luminosity
         """
 
         raise RuntimeError("Must be implemented in derived class")
 
     @abc.abstractmethod
     def draw_luminosity(self, size):
+        """
+        function to draw the luminosity via an alternative method
+        must be implemented in child class
+
+        :param size: 
+        :type size: 
+        :returns: 
+
+        """
         pass
