@@ -11,14 +11,19 @@ class ParetoDistribution(LuminosityDistribution):
 
     def __init__(self, seed: int = 1234, name: str = "pareto"):
         """
-        A Pareto luminosity function
-        :param seed:
-        :param name:
-        :returns:
-        :rtype:
+        A Pareto luminosity function.
 
+        ``alpha``*``Lmin``^``alpha`` / L^(``alpha``+1)
+
+        :param seed: Random seed
+        :type seed: int
+        :param name: Name of the distribution
+        :type name: str
+        :param Lmin: Minimum value of the luminosity
+        :type Lmin: :class:`DistributionParameter`
+        :param alpha: Index of the pareto distribution
+        :type alpha: :class:`DistributionParameter`
         """
-
         lf_form = r"\frac{\alpha L_{\rm min}^{\alpha}}{L^{\alpha+1}}"
 
         super(ParetoDistribution, self).__init__(seed=seed,
@@ -26,14 +31,6 @@ class ParetoDistribution(LuminosityDistribution):
                                                  form=lf_form)
 
     def phi(self, L):
-        """
-        The luminosity function
-
-        :param L:
-        :returns:
-        :rtype:
-
-        """
 
         out = np.zeros_like(L)
 
@@ -45,13 +42,5 @@ class ParetoDistribution(LuminosityDistribution):
         return out
 
     def draw_luminosity(self, size: int = 1):
-        """
-        draw the luminosity
-
-        :param size:
-        :returns:
-        :rtype:
-
-        """
 
         return (np.random.pareto(self.alpha, size) + 1) * self.Lmin
