@@ -14,23 +14,41 @@ class SpiralGalaxyDistribution(SphericalDistribution):
     R1 = DistributionParameter()
     R0 = DistributionParameter()
 
-    def __init__(self, seed=1234, name="spiral_galaxy", form=None):
-
+    def __init__(
+        self,
+        seed: int = 1234,
+        name: str = "spiral_galaxy",
+        form: str = None,
+    ):
         """
-        A spiral galaxy spatial distribution
-
-        :param seed: 
-        :type seed: 
-        :param name: 
-        :type name: 
-        :param form: 
-        :type form: 
-        :returns: 
+        A spiral galaxy spatial distribution.
 
+        :param seed: Random seed
+        :type seed: int
+        :param name: Name of the distribution
+        :type name: str
+        :param form: Mathematical description of distribution
+        :type form: str
+        :param rho: Local density
+        :type rho: :class:`DistributionParameter`
+        :param a: Shape parameter
+        :type a: :class:`DistributionParameter`
+        :param b: Shape parameter
+        :type b: :class:`DistributionParameter`
+        :param R1: Scale parameter
+        :type R1: :class:`DistributionParameter`
+        :param R0: Scale parameter
+        :type R0: :class:`DistributionParameter`
         """
-        super(SpiralGalaxyDistribution, self).__init__(seed=seed,
-                                                       name=name,
-                                                       form=form)
+
+        spatial_form = r"\rho \Big(\frac{r + R_1}{R_0+R_1}\Big)^a "
+        spatial_form += r"\exp\Big[-b \frac{r-R_0}{R_0 + R_1}\Big]"
+
+        super(SpiralGalaxyDistribution, self).__init__(
+            seed=seed,
+            name=name,
+            form=form,
+        )
 
     def dNdV(self, r):
 
@@ -40,11 +58,15 @@ class SpiralGalaxyDistribution(SphericalDistribution):
         ) * np.exp(-self.b * (r - self.R0) / (self.R0 + self.R1)))
 
     def draw_sky_positions(self, size):
+        """
+        Based on Wainscoat 1992 and
+        Faucher-Giguere 2007.
 
-        # code thanks to Moritz!
+        Code thanks to Mortiz Pleintinger.
+        """
 
-        k = np.array([4.25, 4.25, 4.89, 4.89
-                      ])  # Parameter fuer possun [-8.5,0,0], von Wainscoat1992
+        # Parameters for possun [-8.5,0,0], from Wainscoat 1992
+        k = np.array([4.25, 4.25, 4.89, 4.89])
         r0 = np.array([3.48, 3.48, 4.90, 4.90])
         theta0 = np.array([0.0, 3.141, 2.525, 5.666])  # Wainscoat 1992
         height = 0.3  # kpc
@@ -64,8 +86,6 @@ class SpiralGalaxyDistribution(SphericalDistribution):
 
         spiraltheta = tet + corrtet  # Faucher-Giguere 2007
 
-        spiralthetaindec = spiraltheta * 180 / np.pi
-
         zpos = rd.exponential(height, size=size)
 
         zpos *= np.random.choice([-1, 1], size=size)
@@ -76,4 +96,5 @@ class SpiralGalaxyDistribution(SphericalDistribution):
         phi = np.arccos(zpos / np.sqrt(self._distances**2 + zpos**2))
 
         self._theta = spiraltheta
+
         self._phi = phi
