@@ -1,9 +1,7 @@
-import importlib
-from typing import Any, Dict, List, Optional, Type
+from typing import Any, Dict, Optional
 
 import h5py
 import ipyvolume as ipv
-import ipywidgets as widgets
 import matplotlib.pyplot as plt
 import networkx as nx
 import numpy as np
@@ -13,11 +11,14 @@ from betagen import betagen
 from IPython.display import Markdown, Math, display
 from numpy.typing import ArrayLike
 
+from popsynth.auxiliary_sampler import SecondaryStorage
 from popsynth.utils.array_to_cmap import array_to_cmap
 from popsynth.utils.hdf5_utils import (
-    clean_graph_dict, fill_graph_dict,
+    clean_graph_dict,
+    fill_graph_dict,
     recursively_load_dict_contents_from_group,
-    recursively_save_dict_contents_to_group)
+    recursively_save_dict_contents_to_group,
+)
 from popsynth.utils.logging import setup_logger
 from popsynth.utils.spherical_geometry import xyz
 
@@ -29,57 +30,84 @@ dark, dark_highlight, mid, mid_highlight, light, light_highlight = betagen(
 
 
 class Population(object):
-    def __init__(self,
-                 luminosities: ArrayLike,
-                 distances: ArrayLike,
-                 known_distances: ArrayLike,
-                 known_distance_idx: ArrayLike,
-                 unknown_distance_idx: ArrayLike,
-                 fluxes: ArrayLike,
-                 flux_obs: ArrayLike,
-                 selection: ArrayLike,
-                 flux_sigma: float,
-                 r_max: float,
-                 n_model: int,
-                 lf_params: Dict[str, Any],
-                 spatial_params: Optional[Dict[str, Any]] = None,
-                 model_spaces: Optional[ArrayLike] = None,
-                 seed: int = 1234,
-                 name: Optional[str] = None,
-                 spatial_form: Optional[Dict[str, Any]] = None,
-                 lf_form: Optional[Dict[str, Any]] = None,
-                 auxiliary_quantities: Optional[Dict[str, Any]] = None,
-                 truth: Dict[str, float] = {},
-                 graph: Optional[Dict[str, Any]] = None,
-                 theta=None,
-                 phi=None,
-                 pop_synth: Optional[Dict[str, Any]] = None) -> None:
+    def __init__(
+        self,
+        luminosities: ArrayLike,
+        distances: ArrayLike,
+        known_distances: ArrayLike,
+        known_distance_idx: ArrayLike,
+        unknown_distance_idx: ArrayLike,
+        fluxes: ArrayLike,
+        flux_obs: ArrayLike,
+        selection: ArrayLike,
+        flux_sigma: float,
+        r_max: float,
+        n_model: int,
+        lf_params: Dict[str, Any],
+        spatial_params: Optional[Dict[str, Any]] = None,
+        model_spaces: Optional[ArrayLike] = None,
+        seed: int = 1234,
+        name: Optional[str] = None,
+        spatial_form: Optional[Dict[str, Any]] = None,
+        lf_form: Optional[Dict[str, Any]] = None,
+        auxiliary_quantities: Optional[SecondaryStorage] = None,
+        truth: Dict[str, float] = {},
+        graph: Optional[Dict[str, Any]] = None,
+        theta=None,
+        phi=None,
+        pop_synth: Optional[Dict[str, Any]] = None,
+    ) -> None:
         """
         A population container for all the properties of the population.
 
-        :param luminosities: the luminosities
-        :param distances: the distances
-        :param known_distances: the known distances
-        :param known_distance_idx: the index of the known distances
-        :param unknown_distance_idx:  the index of the unknown distances
-        :param fluxes: the latent fluxes
-        :param flux_obs: the observed fluxes
-        :param selection: the selection vector
-        :param flux_sigma: the uncertainty on the observed flux
-        :param r_max: the maximum distance of the survey
-        :param n_model:
-        :param lf_params:
-        :param spatial_params:
-        :param model_spaces:
-        :param seed: the random seed
-        :param name:
-        :param spatial_form:
-        :param lf_form:
-        :param auxiliary_quantities:
-        :param truth:
-        :returns:
-        :rtype:
-
+        :param luminosities: The luminosities
+        :type luminosities: ArrayLike
+        :param distances: The distances
+        :type distances: ArrayLike
+        :param known_distances: The known distances
+        :type known_distances: ArrayLike
+        :param known_distance_idx: The index of the known distances
+        :type known_distance_idx: ArrayLike
+        :param unknown_distance_idx: The index of the unknown distances
+        :type unknown_distance_idx: ArrayLike
+        :param fluxes: The latent fluxes
+        :type fluxes: ArrayLike
+        :param flux_obs: The observed fluxes
+        :type flux_obs: ArrayLike
+        :param selection: The selection vector
+        :type selection: ArrayLike
+        :param flux_sigma: The uncertainty on the observed flux
+        :type flux_sigma: float
+        :param r_max: The maximum distance of the survey
+        :type r_max: float
+        :param n_model: Number of models
+        :type n_model: int
+        :param lf_params: Luminosity function parameters
+        :type lf_params: Dict[str, Any]
+        :param spatial_params: Spatial distribution parameters
+        :type spatial_params: Optional[Dict[str, Any]]
+        :param model_spaces: Model spaces
+        :type model_spaces: ArrayLike
+        :param seed: Random seed
+        :type seed: int
+        :param name: Name of the population
+        :type name: str
+        :param spatial_form: Form of the spatial distribution
+        :type spatial_form: Optional[Dict[str, Any]]
+        :param lf_form: Form of the luminosity function
+        :type lf_form: Optional[Dict[str, Any]]
+        :param auxiliary_quantities: Dict of auxiliary quantities
+        :type auxiliary_quantities: Optional[Dict[str, Any]]
+        :param truth: Dict of true values
+        :type truth: Dict[str, float]
+        :param graph: Graph of generative model
+        :type graph: Optional[Dict[str, Any]]
+        :param theta: Theta
+        :type theta:
+        :param phi: Phi
+        :type phi:
+        :param pop_synth: Population synth
+        :type pop_synth: Optional[Dict[str, Any]]
         """
         self._luminosities = luminosities  # type: ArrayLike
 
@@ -160,6 +188,13 @@ class Population(object):
 
     @property
     def graph(self):
+        """
+        The networkx graph of the population
+
+        :returns: 
+
+        """
+
         return self._graph
 
     @property
@@ -172,11 +207,7 @@ class Population(object):
     @property
     def truth(self):
         """
-        the simulated truth parameters
-
-        :returns:
-        :rtype:
-
+        The simulated truth parameters
         """
 
         return self._truth
@@ -184,51 +215,36 @@ class Population(object):
     @property
     def flux_sigma(self) -> float:
         """
-        the simulated flux sigma
-
-        :returns: 
-
+        The simulated flux sigma
         """
-        
+
         return self._flux_sigma
 
     @property
     def theta(self) -> np.ndarray:
         """
-        the polar angle of the objects
-
-        :returns: 
-
+        The polar angle of the objects
         """
         return self._theta
 
     @property
     def phi(self) -> np.ndarray:
         """
-        the phi angle of the objects
-
-        :returns: 
-
+        The phi angle of the objects
         """
         return self._phi
 
     @property
     def dec(self) -> np.ndarray:
         """
-        the declenation of the objects
-
-        :returns: 
-
+        The declination of the objects
         """
         return 90 - np.rad2deg(self._theta)
 
     @property
     def ra(self) -> np.ndarray:
         """
-        the right ascension of the objects
-
-        :returns: 
-
+        The right ascension of the objects
         """
         return np.rad2deg(self._phi)
 
@@ -236,7 +252,7 @@ class Population(object):
     def luminosities_latent(self) -> np.ndarray:
         """
         The true luminosities of the objects. These are always latent
-        as one cannot directly observe them
+        as one cannot directly observe them.
         """
         return self._luminosities
 
@@ -274,7 +290,6 @@ class Population(object):
         """
         All of the observed fluxes, i.e.,
         scattered with error
-
         """
         return self._flux_obs
 
@@ -298,7 +313,7 @@ class Population(object):
     def selected_distances(self) -> np.ndarray:
         """
         The selected distances. Note, this is different than
-        the KNOWN distances
+        the KNOWN distances.
         """
         return self._distance_selected
 
@@ -371,6 +386,13 @@ class Population(object):
 
     @property
     def spatial_parameters(self):
+        """
+        spatial parameters
+
+        :returns: 
+
+        """
+
         return self._spatial_params
 
     def __repr__(self):
@@ -427,12 +449,10 @@ class Population(object):
 
     def writeto(self, file_name: str) -> None:
         """
-        write population to an HDF5 file
-
-        :param file_name:
-        :returns:
-        :rtype:
+        Write population to an HDF5 file
 
+        :param file_name: Name of the file
+        :type file_name: str
         """
 
         with h5py.File(file_name, "w") as f:
@@ -441,8 +461,13 @@ class Population(object):
 
     def addto(self, file_name: str, group_name: str) -> None:
         """
-        write population to a group in an existing
+        Write population to a group in an existing
         HDF5 file.
+
+        :param file_name: Name of the file
+        :type file_name: str
+        :param group_name: Name of the group
+        :type group_name: str
         """
 
         with h5py.File(file_name, "r+") as f:
@@ -453,7 +478,7 @@ class Population(object):
 
     def _writeto(self, f):
         """
-        write to file or group
+        Write to an HDF5 file or group.
 
         :param f: h5py file or group handle
         """
@@ -535,18 +560,16 @@ class Population(object):
         recursively_save_dict_contents_to_group(
             f, "graph", fill_graph_dict(nx.to_dict_of_dicts(self._graph)))
 
+        # now store the popsynth
         recursively_save_dict_contents_to_group(f, "popsynth", self._pop_synth)
 
-        # now store the popsymnth
-
     @classmethod
-    def from_file(cls, file_name):
+    def from_file(cls, file_name: str):
         """
-        load a population from a file
-        :param file_name:
-        :returns:
-        :rtype:
+        Load a population from a file.
 
+        :param file_name: Name of the file
+        :type file_name: str
         """
 
         with h5py.File(file_name, "r") as f:
@@ -554,10 +577,15 @@ class Population(object):
             return cls._loadfrom(f)
 
     @classmethod
-    def from_group(cls, file_name, group_name):
+    def from_group(cls, file_name: str, group_name: str):
         """
-        load a population from a group
+        Load a population from a group
         in a file.
+
+        :param file_name: Name of the file
+        :type file_name: str
+        :param group_name: Name of the group
+        :type group_name: str
         """
 
         with h5py.File(file_name, "r") as f:
@@ -569,7 +597,7 @@ class Population(object):
     @classmethod
     def _loadfrom(cls, f):
         """
-        load from file or group
+        Load from and HDF5 file or group.
 
         :param f: h5py file or group handle
         """
@@ -652,38 +680,42 @@ class Population(object):
 
         pop_synth = recursively_load_dict_contents_from_group(f, "popsynth")
 
-        return cls(luminosities=luminosities,
-                   distances=distances,
-                   known_distances=known_distances,
-                   known_distance_idx=known_distance_idx,
-                   unknown_distance_idx=unknown_distance_idx,
-                   fluxes=fluxes,
-                   flux_obs=flux_obs,
-                   selection=selection,
-                   flux_sigma=flux_sigma,
-                   n_model=n_model,
-                   r_max=r_max,
-                   lf_params=lf_params,
-                   spatial_params=spatial_params,
-                   model_spaces=model_spaces,
-                   seed=seed,
-                   name=name,
-                   spatial_form=spatial_form,
-                   lf_form=lf_form,
-                   auxiliary_quantities=auxiliary_quantities,
-                   truth=truth,
-                   graph=graph,
-                   theta=theta,
-                   phi=phi,
-                   pop_synth=pop_synth)
+        return cls(
+            luminosities=luminosities,
+            distances=distances,
+            known_distances=known_distances,
+            known_distance_idx=known_distance_idx,
+            unknown_distance_idx=unknown_distance_idx,
+            fluxes=fluxes,
+            flux_obs=flux_obs,
+            selection=selection,
+            flux_sigma=flux_sigma,
+            n_model=n_model,
+            r_max=r_max,
+            lf_params=lf_params,
+            spatial_params=spatial_params,
+            model_spaces=model_spaces,
+            seed=seed,
+            name=name,
+            spatial_form=spatial_form,
+            lf_form=lf_form,
+            auxiliary_quantities=auxiliary_quantities,
+            truth=truth,
+            graph=graph,
+            theta=theta,
+            phi=phi,
+            pop_synth=pop_synth,
+        )
 
     def to_sub_population(self, observed: bool = True) -> "Population":
         """
         Create a population that is down selected from either the
         observed or unobserved population
 
-        :param observed: extract the observed or unobserved object
-
+        :param observed: Extract the observed or unobserved object
+        :type observed: bool
+        :returns: A new population object
+        :rtype: :class:`Population`
         """
 
         if observed:
@@ -756,8 +788,7 @@ class Population(object):
 
     def display(self):
         """
-        Display the simulation parameters
-
+        Display the simulation parameters.
         """
 
         info = "### %s simulation\nDetected %d out of %d objects" % (
@@ -793,13 +824,11 @@ class Population(object):
         display(pd.DataFrame(out))
 
     def display_true_fluxes(self, ax=None, flux_color=dark, **kwargs):
-        """Display the fluxes
-
-        :param ax:
-        :param flux_color:
-        :returns:
-        :rtype:
+        """
+        Display the fluxes.
 
+        :param ax: Axis on which to plot
+        :param flux_color: Color of fluxes
         """
 
         if ax is None:
@@ -839,13 +868,10 @@ class Population(object):
 
     def display_obs_fluxes(self, ax=None, flux_color=dark, **kwargs):
         """
-        display the observed fluxes
-
-        :param ax:
-        :param flux_color:
-        :returns:
-        :rtype:
+        Display the observed fluxes.
 
+        :param ax: Axis on which to plot
+        :param flux_color: Color of fluxes
         """
 
         # do not try to plot if there is nothing
@@ -901,16 +927,13 @@ class Population(object):
         **kwargs,
     ):
         """
-        display the fluxes
-
-        :param ax:
-        :param true_color:
-        :param obs_color:
-        :param arrow_color:
-        :param with_arrows:
-        :returns:
-        :rtype:
+        Display the fluxes.
 
+        :param ax: Axis on which to plot
+        :param true_color: Color of true values
+        :param obs_color: Color of obs values
+        :param arrow_color: Color of arrows
+        :param with_arrows: If `True`, display arrows
         """
 
         if ax is None:
@@ -958,15 +981,11 @@ class Population(object):
         **kwargs,
     ):
         """
-        display the luminosities
-
-        :param ax:
-        :param true_color:
-        :param obs_color:
-
-        :returns:
-        :rtype:
+        Display the luminosities
 
+        :param ax: Axis on which to plot
+        :param true_color: Color of true values
+        :param obs_color: Color of obs values
         """
 
         if ax is None:
@@ -1040,7 +1059,7 @@ class Population(object):
             fig.controls = control
             control.autoRotate = True
             fig.render_continuous = True
-            #control.autoRotate = True
+            # control.autoRotate = True
             # toggle_rotate = widgets.ToggleButton(description="Rotate")
             # widgets.jslink((control, "autoRotate"), (toggle_rotate, "value"))
             # r_value = toggle_rotate
@@ -1142,12 +1161,9 @@ class Population(object):
 
     def display_distances(self, ax=None):
         """
-        display the distances
-
-        :param ax:
-        :returns:
-        :rtype:
+        Display the distances
 
+        :param ax: Axis on which to plot
         """
 
         if ax is None:
