@@ -1,4 +1,4 @@
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
 from collections import OrderedDict
 from typing import Any, Dict, Optional, Union
 
@@ -12,15 +12,17 @@ from IPython.display import Markdown, Math, display
 
 # from numpy.typing import np.ndarray
 
-
-from popsynth.auxiliary_sampler import AuxiliarySampler, DerivedLumAuxSampler
+from popsynth.auxiliary_sampler import (
+    AuxiliarySampler,
+    DerivedLumAuxSampler,
+    SecondaryContainer,
+    SecondaryStorage,
+)
 from popsynth.distribution import LuminosityDistribution, SpatialDistribution
 from popsynth.distributions.cosmological_distribution import CosmologicalDistribution
 from popsynth.population import Population
-from popsynth.selection_probability import ( SelectionProbabilty,
-                                             UnitySelection)
+from popsynth.selection_probability import SelectionProbability, UnitySelection
 from popsynth.utils.logging import setup_logger
-from popsynth.utils.progress_bar import progress_bar
 from popsynth.utils.registry import (
     auxiliary_parameter_registry,
     distribution_registry,
@@ -42,11 +44,11 @@ class PopulationSynth(object, metaclass=ABCMeta):
         derived luminosity distribution and everything is setup.
 
         :param spatial_distribution: The spatial distribution to sample locations from
-        :param luminosity_distribution: the optional luminosity distribution
-        :param seed: random see
-        :returns:
-        :rtype:
-
+        :type spatial_distribution: :class:`SpatialDistribution`
+        :param luminosity_distribution: The optional luminosity distribution
+        :type luminosity_distribution: :class:`LuminosityDistribution`
+        :param seed: Random seed
+        :type seed: int
         """
 
         self._n_model = 500  # type: int
@@ -88,15 +90,15 @@ class PopulationSynth(object, metaclass=ABCMeta):
             None)  # type: Union[DerivedLumAuxSampler, None]
 
         # set the selections be fully seen unless it is set by the user
-        self._distance_selector: SelectionProbabilty = UnitySelection(
+        self._distance_selector: SelectionProbability = UnitySelection(
             name="unity distance selector")
-        self._flux_selector: SelectionProbabilty = UnitySelection(
+        self._flux_selector: SelectionProbability = UnitySelection(
             name="unity flux selector")
 
         # check to see if the selectors are set
         self._distance_selector_set: bool = False
         self._flux_selector_set: bool = False
-        self._spatial_selector: Optional[SelectionProbabilty] = None
+        self._spatial_selector: Optional[SelectionProbability] = None
 
         self._params = {}  # type: dict
 
@@ -117,10 +119,11 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def clean(self, reset: bool = False):
         """
-        clean the auxiliary samplers, selections, etc
+        Clean the auxiliary samplers, selections, etc
         from the population synth
-        :param reset: reset any attached distributions and samplers
 
+        :param reset: If `True`, reset any attached distributions and samplers
+        :type reset: bool
         """
 
         if reset:
@@ -176,9 +179,10 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def write_to(self, file_name: str) -> None:
         """
-        write the population synth to a file
-        :param file_name: the file name of the output yaml
+        Write the population synth to a YAML file.
 
+        :param file_name: the file name of the output YAML
+        :type file_name: str
         """
         with open(file_name, "w") as f:
 
@@ -191,7 +195,10 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def to_dict(self) -> Dict[str, Any]:
         """
-        convert the population synth to a dictionary
+        Convert the population synth to a dictionary
+
+        :returns: Popsynth dict
+        :rtype: Dict[str, Any]
         """
 
         output: Dict[str, Any] = {}
@@ -290,9 +297,12 @@ class PopulationSynth(object, metaclass=ABCMeta):
     @classmethod
     def from_dict(cls, input: Dict[str, Any]) -> "PopulationSynth":
         """
-        build a PopulationSynth object from a dictionary
+        Build a PopulationSynth object from a dictionary
 
         :param input: the dictionary from which to build
+        :type input: Dict[str, Any]
+        :returns: Popsynth object
+        :rtype: :class:`PopulationSynth`
         """
 
         if "luminosity distribution" in input:
@@ -307,7 +317,7 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
             # create the instance
 
-            luminosity_distribtuion: LuminosityDistribution = distribution_registry[
+            luminosity_distribution: LuminosityDistribution = distribution_registry[
                 ld_name]
 
             # now set the values of the parameters
@@ -318,19 +328,19 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
                 log.debug(f"trying to set {k} to {v}")
 
-                for x in luminosity_distribtuion.__class__.mro():
+                for x in luminosity_distribution.__class__.mro():
 
                     if k in x.__dict__:
 
-                        setattr(luminosity_distribtuion, k, float(v))
+                        setattr(luminosity_distribution, k, float(v))
 
                         break
 
-                log.debug(f"{luminosity_distribtuion.params}")
+                log.debug(f"{luminosity_distribution.params}")
 
         else:
 
-            luminosity_distribtuion = None
+            luminosity_distribution = None
 
         # try
 
@@ -363,7 +373,7 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
         pop_synth: PopulationSynth = cls(
             spatial_distribution,
-            luminosity_distribution=luminosity_distribtuion,
+            luminosity_distribution=luminosity_distribution,
             seed=seed,
         )
 
@@ -719,11 +729,10 @@ class PopulationSynth(object, metaclass=ABCMeta):
         """
         Add a model space for stan generated quantities
 
-        :param name: name that Stan will use
-        :param start: start of the grid
-        :param stop: stop of the grid
-        :param log: use log10 or not
-
+        :param name: Name that Stan will use
+        :param start: Start of the grid
+        :param stop: Stop of the grid
+        :param log: Use log10 or not
         """
         if log:
             space = np.logspace(np.log10(start), np.log10(stop), self._n_model)
@@ -738,12 +747,11 @@ class PopulationSynth(object, metaclass=ABCMeta):
                               auxiliary_sampler: Union[DerivedLumAuxSampler,
                                                        AuxiliarySampler]):
         """
-        add an auxiliary sampler or derived luminosity sampler to the population
-        synth
+        Add an auxiliary sampler or derived luminosity sampler to the population
+        synth.
 
-        :param auxiliary_sampler:
-        :returns:
-        :rtype:
+        :param auxiliary_sampler: The auxiliary_sampler
+        :type auxiliary_sampler: Union[DerivedLumAuxSampler, AuxiliarySampler]
         """
 
         self.add_observed_quantity(auxiliary_sampler)
@@ -752,13 +760,11 @@ class PopulationSynth(object, metaclass=ABCMeta):
                               auxiliary_sampler: Union[DerivedLumAuxSampler,
                                                        AuxiliarySampler]):
         """
-        add an auxiliary sampler or derived luminosity sampler to the population
+        Add an auxiliary sampler or derived luminosity sampler to the population
         synth
 
-        :param auxiliary_sampler:
-        :returns:
-        :rtype:
-
+        :param auxiliary_sampler: The auxiliary_sampler
+        :type auxiliary_sampler: Union[DerivedLumAuxSampler, AuxiliarySampler]
         """
 
         if isinstance(auxiliary_sampler, DerivedLumAuxSampler):
@@ -799,12 +805,15 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
             raise RuntimeError()
 
-    def set_distance_selection(self, selector: SelectionProbabilty) -> None:
+    def set_distance_selection(self, selector: SelectionProbability) -> None:
         """
-        Set the selection type for the distance
+        Set the selection type for the distance.
+
+        :param selector: The selector
+        :type selector: :class:`SelectionProbability`
         """
 
-        if not isinstance(selector, SelectionProbabilty):
+        if not isinstance(selector, SelectionProbability):
 
             log.error(f"{selector} is not a Selection probability")
 
@@ -814,11 +823,14 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
         self._distance_selector_set = True
 
-    def set_flux_selection(self, selector: SelectionProbabilty) -> None:
+    def set_flux_selection(self, selector: SelectionProbability) -> None:
         """
-        Set the selection type for the distance
+        Set the selection type for the flux
+
+        :param selector: The selector
+        :type selector: :class:`SelectionProbability`
         """
-        if not isinstance(selector, SelectionProbabilty):
+        if not isinstance(selector, SelectionProbability):
 
             log.error(f"{selector} is not a Selection probability")
 
@@ -829,12 +841,15 @@ class PopulationSynth(object, metaclass=ABCMeta):
         self._flux_selector_set = True
 
     def add_spatial_selector(self,
-                             spatial_selector: SelectionProbabilty) -> None:
+                             spatial_selector: SelectionProbability) -> None:
         """
         Add a spatial selector into the mix
+
+        :param spatial_selector: The spatial selector
+        :type spatial_selector: :class:`SelectionProbability`
         """
 
-        if not isinstance(spatial_selector, SelectionProbabilty):
+        if not isinstance(spatial_selector, SelectionProbability):
 
             log.error(f"{spatial_selector} is not a Selection probability")
 
@@ -848,7 +863,7 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def draw_log10_fobs(self, f, f_sigma, size=1) -> np.ndarray:
         """
-        draw the log10 of the the fluxes
+        Draw the log10 of the the fluxes.
         """
 
         log10_f = np.log10(f)
@@ -861,7 +876,7 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def draw_log_fobs(self, f, f_sigma, size=1) -> np.ndarray:
         """
-        draw the log10 of the the fluxes
+        Draw the log10 of the the fluxes.
         """
 
         log_f = np.log(f)
@@ -877,17 +892,19 @@ class PopulationSynth(object, metaclass=ABCMeta):
         log10_flux_draw: bool = True,
     ) -> Population:
         """
-        Draw the total survey and return a Population object.
-        
-        This will sample all attached distributions and apply selection functions.
-        
-        If a value of flux_sigma is given, the log10 observed fluxes are sampled with 
-        measurement error.
-        
+        Draw the total survey and return a :class:`Population` object.
 
-        :param flux_sigma: the homoskedastic sigma for the flux in log10 space
-        :param log10_flux_draw: if True, fluxes are drawn in log space
+        This will sample all attached distributions and apply selection functions.
+
+        If a value of flux_sigma is given, the log10 observed fluxes are sampled with
+        measurement error.
+
+        :param flux_sigma: The homoskedastic sigma for the flux in log10 space
+        :type flux_sigma: Optional[float]
+        :param log10_flux_draw: if `True`, fluxes are drawn in log space
+        :type log10_flux_draw: bool
         :return: a Population object
+        :rtype: :class:`Population`
         """
 
         # this stores all the "true" population values from all the samplers
@@ -931,17 +948,17 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
         # setup the global selection
 
-        global_selection: SelectionProbabilty = UnitySelection(name="global")
+        global_selection: SelectionProbability = UnitySelection(name="global")
         global_selection.select(n)
 
         # now we set up the selection that _may_ come
         # from the auxilliary samplers
 
-        auxiliary_selection: SelectionProbabilty = UnitySelection(
+        auxiliary_selection: SelectionProbability = UnitySelection(
             name="total auxiliary selection")
         auxiliary_selection.select(n)
 
-        auxiliary_quantities: Dict[str, Dict] = {}
+        auxiliary_quantities: SecondaryStorage = SecondaryStorage()
 
         # this means the luminosity is not
         # simulated directy
@@ -978,11 +995,14 @@ class PopulationSynth(object, metaclass=ABCMeta):
                     and len(self._derived_luminosity_sampler.obs_values) == n)
 
             # append these values to a dict
-            auxiliary_quantities[self._derived_luminosity_sampler.name] = {
-                "true_values": self._derived_luminosity_sampler.true_values,
-                "obs_values": self._derived_luminosity_sampler.obs_values,
-                "selection": self._derived_luminosity_sampler.selector,
-            }
+
+            auxiliary_quantities.add_secondary(
+                SecondaryContainer(
+                    self._derived_luminosity_sampler.name,
+                    self._derived_luminosity_sampler.true_values,
+                    self._derived_luminosity_sampler.obs_values,
+                    self._derived_luminosity_sampler.selector,
+                ))
 
             log.info("Getting luminosity from derived sampler")
 
@@ -1005,12 +1025,14 @@ class PopulationSynth(object, metaclass=ABCMeta):
                 # first we tell the sampler to go and retrieve all of
                 # its own secondaries
 
-                properties = v2.get_secondary_properties()  # type: dict
+                auxiliary_quantities += v2.get_secondary_properties()
 
-                for k3, v3 in properties.items():
+                #                properties = v2.get_secondary_properties()  # type: dict
 
-                    # now attach them
-                    auxiliary_quantities[k3] = v3
+                # for k3, v3 in properties.items():
+
+                #     # now attach them
+                #     auxiliary_quantities[k3] = v3
 
                 # store the secondary truths
                 # this will _could_ be clobbered later
@@ -1060,12 +1082,14 @@ class PopulationSynth(object, metaclass=ABCMeta):
             assert v.true_values is not None and len(v.true_values) == n
             assert v.obs_values is not None and len(v.obs_values) == n
 
-            # append these values to a dict
-            auxiliary_quantities[k] = {
-                "true_values": v.true_values,
-                "obs_values": v.obs_values,
-                "selection": v.selector,
-            }  # type: dict
+            auxiliary_quantities += v.get_secondary_properties()
+
+            # # append these values to a dict
+            # auxiliary_quantities[k] = {
+            #     "true_values": v.true_values,
+            #     "obs_values": v.obs_values,
+            #     "selection": v.selector,
+            # }  # type: dict
 
             # collect the secondary values
 
@@ -1074,12 +1098,12 @@ class PopulationSynth(object, metaclass=ABCMeta):
                 # first we tell the sampler to go and retrieve all of
                 # its own secondaries
 
-                properties = v2.get_secondary_properties()  # type: dict
+                # properties = v2.get_secondary_properties()  # type: dict
 
-                for k3, v3 in properties.items():
+                # for k3, v3 in properties.items():
 
-                    # now attach them
-                    auxiliary_quantities[k3] = v3
+                #     # now attach them
+                #     auxiliary_quantities[k3] = v3
 
                 # store the secondary truths
 
@@ -1249,33 +1273,82 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def display(self) -> None:
         """
-        Display the simulation parameters
-
+        Display the simulation parameters.
         """
 
-        if self._luminosity_distribution is not None:
-
-            out = {"parameter": [], "value": []}
+        if self._has_derived_luminosity:
 
             display(Markdown("## Luminosity Function"))
-            for k, v in self._luminosity_distribution.params.items():
 
-                out["parameter"].append(k)
-                out["value"].append(v)
+            self._derived_luminosity_sampler.display()
 
-            display(Math(self._luminosity_distribution.form))
-            display(pd.DataFrame(out))
-        out = {"parameter": [], "value": []}
+        elif self._luminosity_distribution is not None:
+
+            display(Markdown("## Luminosity Function"))
+
+            self._luminosity_distribution.display()
 
         display(Markdown("## Spatial Function"))
 
-        for k, v in self._spatial_distribution.params.items():
+        self._spatial_distribution.display()
 
-            out["parameter"].append(k)
-            out["value"].append(v)
+        names = []
 
-        display(Math(self._spatial_distribution.form))
-        display(pd.DataFrame(out))
+        if self._has_derived_luminosity:
+
+            for k, v in self._derived_luminosity_sampler.secondary_samplers.items(
+            ):
+
+                names.append(k)
+
+                display(Markdown(f"## {k}"))
+
+                v.display()
+
+        for k, v in self._auxiliary_observations.items():
+
+            if k not in names:
+
+                display(Markdown(f"## {k}"))
+
+                v.display()
+
+    def __repr__(self) -> str:
+
+        if self._has_derived_luminosity:
+
+            out = "Luminosity Function\n"
+
+            out += self._derived_luminosity_sampler.__repr__()
+
+        elif self._luminosity_distribution is not None:
+
+            out = "Luminosity Function\n"
+
+            out += self._luminosity_distribution.__repr__()
+
+        out += "Spatial Function\n"
+
+        out += self._spatial_distribution.__repr__()
+
+        names = []
+
+        if self._has_derived_luminosity:
+
+            for k, v in self._derived_luminosity_sampler.secondary_samplers.items(
+            ):
+
+                names.append(k)
+
+                out += v.__repr__()
+
+        for k, v in self._auxiliary_observations.items():
+
+            if k not in names:
+
+                out += v.__repr__()
+
+        return out
 
     # def generate_stan_code(self, stan_gen, **kwargs):
 
@@ -1290,11 +1363,7 @@ class PopulationSynth(object, metaclass=ABCMeta):
 
     def _build_graph(self):
         """
-        builds the graph for all the samplers
-
-        :returns:
-        :rtype:
-
+        Builds the graph for all the samplers.
         """
 
         # first check out the luminosity sampler
