@@ -1,5 +1,3 @@
-import numpy as np
-
 from popsynth.utils.logging import setup_logger
 
 from .generic_selectors import LowerBound, SoftSelection
@@ -11,6 +9,11 @@ class HardFluxSelection(LowerBound):
     _selection_name = "HardFluxSelection"
 
     def __init__(self) -> None:
+        """
+        A hard selection on the observed flux.
+
+        Based on :class:`LowerBound`.
+        """
 
         super(HardFluxSelection, self).__init__(use_flux=True)
 
@@ -23,6 +26,11 @@ class SoftFluxSelection(SoftSelection):
     _selection_name = "SoftFluxSelection"
 
     def __init__(self) -> None:
+        """
+        A soft selection on the observed flux.
+
+        Based on :class:`SoftSelection`.
+        """
 
         super(SoftFluxSelection, self).__init__(use_flux=True)
 
