@@ -2,16 +2,18 @@ import matplotlib as mpl
 import matplotlib.pyplot as plt
 
 
-def array_to_cmap(values, cmap, use_log=False):
+def array_to_cmap(values, cmap, use_log: bool = False):
     """
     Generates a color map and color list that is normalized
     to the values in an array. Allows for adding a 3rd dimension
-    onto a plot
-
-    :param values: a list a values to map into a cmap
-    :param cmap: the mpl colormap to use
-    :param use_log: if the mapping should be done in log space
+    onto a plot.
 
+    :param values: A list a values to map into a cmap
+    :param cmap: The mpl colormap to use
+    :param use_log: `True` if the mapping should be done in log space
+        `False` by default.
+    :type use_log: bool
+    :returns: A color map and a normalized color list
     """
 
     if use_log:
