@@ -2,6 +2,10 @@ import numpy as np
 
 
 def sample_theta_phi(size: int):
+    """
+    Sample ``size`` samples uniformly on the
+    surface of the unit sphere.
+    """
 
     theta = np.arccos(1 - 2 * np.random.uniform(0.0, 1.0, size=size))
     phi = np.random.uniform(0, 2 * np.pi, size=size)
@@ -10,6 +14,9 @@ def sample_theta_phi(size: int):
 
 
 def xyz(r, theta, phi):
+    """
+    Convert spherical coordinates to Cartesian.
+    """
     x = r * np.sin(theta) * np.cos(phi)
     y = r * np.sin(theta) * np.sin(phi)
     z = r * np.cos(theta)
